{
	description = "Application packaged using poetry2nix";

	inputs.flake-utils.url = "github:numtide/flake-utils";
	inputs.nixpkgs.url = "github:NixOS/nixpkgs";
	inputs.poetry2nixSrc = {
		url = "github:nix-community/poetry2nix";
		inputs.nixpkgs.follows = "nixpkgs";
	};

	outputs = { self, nixpkgs, flake-utils, poetry2nixSrc }:
	flake-utils.lib.eachDefaultSystem (system:
	let
		pkgs = nixpkgs.legacyPackages.${system};
		poetry2nix = poetry2nixSrc.lib.mkPoetry2Nix { inherit pkgs; };
	in {
		packages = {
			kalpaApp = poetry2nix.mkPoetryApplication {
				projectDir = self;
				python = pkgs.python39;
				preferWheels = true;
			};
			kalpaEnv = poetry2nix.mkPoetryEnv {
				projectDir = self;
				python = pkgs.python39;
				preferWheels = true;
				overrides = poetry2nix.overrides.withDefaults (self: super: {
				});
			};
			default = self.packages.${system}.kalpaEnv;
		};
		devShells.default = pkgs.mkShell {
			inputsFrom = [ self.packages.${system}.kalpaEnv ];
			buildInputs = [
				pkgs.poetry
				self.packages.${system}.kalpaEnv
				self.packages.${system}.kalpaApp
				pkgs.just
				pkgs.nodejs
			];
			shellHook = ''
				export DO_NIX_CUSTOM=1
			'';
		};
	}
	);
}
